import { Injectable } from '@angular/core';
import { Actions, Effect, ofType } from '@ngrx/effects';
import { tap, withLatestFrom } from 'rxjs/operators';
import { GlobalState } from '@app/core/state';
import { Store } from '@ngrx/store';
import { DegreePlannerApiService } from '@app/degree-planner/services/api.service';
import {
  UpdateUserPreferences,
  UserPreferencesActionTypes,
} from '@app/degree-planner/store/actions/userPreferences.actions';

@Injectable()
export class UserPreferencesEffects {
  constructor(
    private actions$: Actions,
    private api: DegreePlannerApiService,
    private store$: Store<GlobalState>,
  ) {}
  @Effect({ dispatch: false })
  updateUserPreferences$ = this.actions$.pipe(
    ofType<UpdateUserPreferences>(
      UserPreferencesActionTypes.UpdateUserPreferences,
    ),
    withLatestFrom(this.store$),
    tap(([update, state]) => {
      const prefs = { ...state.degreePlanner.userPreferences };

      // Remove properties who's value is undefined
      Object.entries(update.payload).forEach(([key, value]) => {
        if (value === undefined && prefs[key] !== undefined) {
          delete prefs[key];
        } else {
          prefs[key] = value;
        }
      });

      this.api.updateUserPreferences(prefs).toPromise();
      // Must .toPromise() the API call for it to run
    }),
  );
}
